from bitpapa_pay.methods.base import BaseMethod
from bitpapa_pay.methods.create_telegram_invoice import (CreateTelegramInvoice,
                                                         TelegramInvoice)
from bitpapa_pay.methods.get_exchange_rates_all import GetExchangeRatesAll
from bitpapa_pay.methods.get_telegram_invoices import GetTelegramInvoices
